!define VERSION "2.1.6"
!define REGROOT "Software\Itefix\ICW"
!define NAME "ICW"
!define PACKAGE "Base"
!define UNINSTPROG "$INSTDIR\uninstall_${NAME}_${PACKAGE}.exe"

SetCompressor /SOLID LZMA

!include "MUI.nsh"

Name "${NAME} ${VERSION}"
OutFile "${NAME}_${PACKAGE}_${VERSION}_installer.exe"
InstallDir $PROGRAMFILES\${NAME}
InstallDirRegKey HKLM ${REGROOT} "InstallDirectory"

VIAddVersionKey  "ProductName" "${NAME}"
VIAddVersionKey  "CompanyName" "ITeF!x Consulting"
VIAddVersionKey  "FileDescription" "${NAME} ${PACKAGE}"
VIAddVersionKey  "FileVersion" "${VERSION}"
VIProductVersion "${VERSION}.0"

!insertmacro MUI_PAGE_INSTFILES 
!insertmacro MUI_UNPAGE_INSTFILES

!insertmacro MUI_LANGUAGE "English"

!include "${NSISDIR}\Include\WinMessages.nsh"


Function .onInit
	
	Init_Cont_A:	
	# Check for previous package installations
	ReadRegStr $0 HKLM "${REGROOT}\${PACKAGE}" "version"
	IfErrors Init_End
		
	MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION "${NAME} ${PACKAGE} version $0 is found on this system. \
		An upgrade/repair will be performed." /SD IDOK IDOK Init_End
	Abort
	
	Init_End:
	
FunctionEnd

Function un.onInit
FunctionEnd

# Install section
Section "${NAME} ${PACKAGE}"

	SetAutoClose true
	
	DetailPrint "Installing Files"
	Call InstallFiles
	
	DetailPrint "Create Fstab"
	Call CreateEtcFstab
	
	; Link /bin/sh to bash
	NsExec::Exec '"$INSTDIR\bin\bash" -c "/bin/ln /bin/bash /bin/sh"'
	
	; Write the installation path and version into the registry
	WriteRegStr HKLM ${REGROOT} "InstallDirectory" "$INSTDIR"
	WriteRegStr HKLM "${REGROOT}\${PACKAGE}" "Version" "${VERSION}"

	; Write the uninstall keys for Windows
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${PACKAGE}" "DisplayName" "${NAME} ${PACKAGE}(remove only)"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${PACKAGE}" "UninstallString" '"${UNINSTPROG}"'
	
	WriteUninstaller ${UNINSTPROG}
SectionEnd

# Uninstall section
Section "Uninstall"

	SetAutoClose true

	# Check if it is in use by some other software
	
	DetailPrint "Remove registry keys"
	DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${PACKAGE}"
	DeleteRegKey HKLM ${REGROOT}
	DeleteRegKey /ifempty HKLM "SOFTWARE\Itefix"

	DetailPrint "Remove uninstaller"
	Delete ${UNINSTPROG}
	
	Call un.DeleteFiles
	
SectionEnd

Function CreateEtcFstab

	# Create /etc/fstab here
	CreateDirectory "$INSTDIR\etc"
	
	# convert backslahes to slashes
	StrCpy $0 $INSTDIR
	Push $0
	Push "\"
	Push "/"
	Call StrRep
	Pop $0 ;result
		
	# Convert spaces to \040 (hex) 
	Push $0
	Push " "
	Push "\040"
	Call StrRep
	Pop $0 ;result	

	FileOpen $1 "$INSTDIR\etc\fstab" w
	FileWrite $1 "# fstab file created by ICW Base installer$\r$\n"
	FileWrite $1 "#$\r$\n"
	FileWrite $1 "# see fstab.txt in the installation directory for more information$\r$\n"
	FileWrite $1 "$0 / ntfs override,binary,noacl 0 0$\r$\n"
	FileWrite $1 "$0/bin /usr/bin ntfs override,binary,noacl 0 0$\r$\n"
	FileWrite $1 "$0/lib /usr/lib ntfs override,binary,noacl 0 0$\r$\n"
	FileWrite $1 "$0/etc/terminfo /usr/share/terminfo ntfs override,binary,noacl 0 0$\r$\n"
	FileWrite $1 "none /cygdrive cygdrive binary,posix=0,user,noacl 0 0$\r$\n"
	
	FileClose $0
	
FunctionEnd

Function InstallFiles
	
	SetOutPath "$INSTDIR"
	File fstab.txt
	
	SetOutPath "$INSTDIR\etc\terminfo\c"
	File terminfo\cygwin
	
	SetOutPath "$INSTDIR\bin"
	File bin\bash.exe
	# Force to replace nano
	Delete "$INSTDIR\bin\nano.exe"
	File bin\nano.exe
	File bin\cat.exe
	File bin\chgrp.exe
	File bin\chmod.exe
	File bin\chown.exe
	File bin\cp.exe
	File bin\cygcheck.exe
	File bin\cygcrypt-0.dll
	File bin\cygcrypto-0.9.8.dll
	File bin\cygssl-0.9.8.dll
	File bin\cygiconv-2.dll
	File bin\cygintl-1.dll
	File bin\cygintl-2.dll
	File bin\cygintl-3.dll
	File bin\cygintl-8.dll
	File bin\cygminires.dll
	File bin\cygncurses-8.dll
	File bin\cygncurses-9.dll
	File bin\cygpath.exe
	File bin\cygpcre-0.dll
	File bin\cygpcre.dll
	File bin\cygpopt-0.dll
	File bin\cygreadline7.dll
	File bin\cygwrap-0.dll
	File bin\cygrunsrv.exe
	File bin\cygwin1.dll
	File bin\cygz.dll
	File bin\cygssp-0.dll
	File bin\cyggcc_s-1.dll
	File bin\d2u.exe
	File bin\date.exe
	File bin\diff.exe
	File bin\false.exe
	File bin\grep.exe
	File bin\groups
	File bin\gzip.exe
	File bin\id.exe
	File bin\less.exe
	File bin\ln.exe
	File bin\ln.exe.manifest
	File bin\ls.exe
	File bin\mkdir.exe
	File bin\mkgroup.exe
	File bin\mkpasswd.exe
	File bin\mount.exe
	File bin\mv.exe
	File bin\ps.exe
	File bin\pwd.exe
	File bin\rm.exe
	File bin\sed.exe
	File bin\sort.exe
	File bin\tail.exe
	File bin\tar.exe
	File bin\touch.exe
	File bin\u2d.exe
	File bin\umount.exe
	File bin\wc.exe
	File bin\cygncurses-10.dll
	File bin\cygattr-1.dll
	File bin\cygedit-0.dll
	File bin\basename.exe
	File bin\bc.exe
	File bin\cygreadline6.dll
	File bin\egrep.exe
	File bin\which.exe
	File bin\xxd.exe
		
FunctionEnd

Function un.DeleteFiles

	Delete $INSTDIR\fstab.txt
	Delete $INSTDIR\bin\bash.exe
	Delete $INSTDIR\bin\nano.exe
	Delete $INSTDIR\bin\sh.exe ; was hardlinked
	Delete $INSTDIR\bin\cat.exe
	Delete $INSTDIR\bin\chgrp.exe
	Delete $INSTDIR\bin\chmod.exe
	Delete $INSTDIR\bin\chown.exe
	Delete $INSTDIR\bin\cp.exe
	Delete $INSTDIR\bin\cygcheck.exe
	Delete $INSTDIR\bin\cygpath.exe
	Delete $INSTDIR\bin\d2u.exe
	Delete $INSTDIR\bin\date.exe
	Delete $INSTDIR\bin\diff.exe
	Delete $INSTDIR\bin\false.exe
	Delete $INSTDIR\bin\grep.exe
	Delete $INSTDIR\bin\groups
	Delete $INSTDIR\bin\gzip.exe
	Delete $INSTDIR\bin\id.exe
	Delete $INSTDIR\bin\less.exe
	Delete $INSTDIR\bin\ln.exe
	Delete $INSTDIR\bin\ln.exe.manifest
	Delete $INSTDIR\bin\ls.exe
	Delete $INSTDIR\bin\mkdir.exe
	Delete $INSTDIR\bin\mkgroup.exe
	Delete $INSTDIR\bin\mkpasswd.exe
	Delete $INSTDIR\bin\mount.exe
	Delete $INSTDIR\bin\mv.exe
	Delete $INSTDIR\bin\ps.exe
	Delete $INSTDIR\bin\pwd.exe
	Delete $INSTDIR\bin\rm.exe
	Delete $INSTDIR\bin\sed.exe
	Delete $INSTDIR\bin\sort.exe
	Delete $INSTDIR\bin\tail.exe
	Delete $INSTDIR\bin\tar.exe
	Delete $INSTDIR\bin\touch.exe
	Delete $INSTDIR\bin\u2d.exe
	Delete $INSTDIR\bin\umount.exe
	Delete $INSTDIR\bin\wc.exe
	Delete $INSTDIR\bin\cygrunsrv.exe
	
	Delete $INSTDIR\cygssp-0.dll
	Delete $INSTDIR\cyggcc_s-1.dll
	Delete $INSTDIR\bin\cygz.dll
	Delete $INSTDIR\bin\cygwrap-0.dll	
	Delete $INSTDIR\bin\cygssl-0.9.8.dll
	Delete $INSTDIR\bin\cygcrypto-0.9.8.dll
	Delete $INSTDIR\bin\cygpopt-0.dll
	Delete $INSTDIR\bin\cygpcre.dll
	Delete $INSTDIR\bin\cygpcre-0.dll
	Delete $INSTDIR\bin\cygminires.dll	
	Delete $INSTDIR\bin\cygreadline7.dll
	Delete $INSTDIR\bin\cygncurses-8.dll
	Delete $INSTDIR\bin\cygncurses-9.dll
	Delete $INSTDIR\bin\cygintl-8.dll	
	Delete $INSTDIR\bin\cygintl-3.dll
	Delete $INSTDIR\bin\cygintl-2.dll
	Delete $INSTDIR\bin\cygintl-1.dll	
	Delete $INSTDIR\bin\cygcrypt-0.dll
	Delete $INSTDIR\bin\cygiconv-2.dll
	Delete $INSTDIR\bin\cygwin1.dll
	
	Delete $INSTDIR\bin\cygncurses-10.dll
	Delete $INSTDIR\bin\cygattr-1.dll
	Delete $INSTDIR\bin\cygedit-0.dll
	
	Delete $INSTDIR\bin\basename.exe
	Delete $INSTDIR\bin\bc.exe
	Delete $INSTDIR\bin\cygreadline6.dll
	Delete $INSTDIR\bin\egrep.exe
	Delete $INSTDIR\bin\which.exe
	Delete $INSTDIR\bin\xxd.exe

	; Remove installation directory if empty
	RMDir /r $INSTDIR\etc\terminfo
	RMDir $INSTDIR\bin
	RMDir $INSTDIR\tmp ; Created during ln sh command
	RMDir $INSTDIR
FunctionEnd

Function StrRep
  Exch $R4 ; $R4 = Replacement String
  Exch
  Exch $R3 ; $R3 = String to replace (needle)
  Exch 2
  Exch $R1 ; $R1 = String to do replacement in (haystack)
  Push $R2 ; Replaced haystack
  Push $R5 ; Len (needle)
  Push $R6 ; len (haystack)
  Push $R7 ; Scratch reg
  StrCpy $R2 ""
  StrLen $R5 $R3
  StrLen $R6 $R1
loop:
  StrCpy $R7 $R1 $R5
  StrCmp $R7 $R3 found
  StrCpy $R7 $R1 1 ; - optimization can be removed if U know len needle=1
  StrCpy $R2 "$R2$R7"
  StrCpy $R1 $R1 $R6 1
  StrCmp $R1 "" done loop
found:
  StrCpy $R2 "$R2$R4"
  StrCpy $R1 $R1 $R6 $R5
  StrCmp $R1 "" done loop
done:
  StrCpy $R3 $R2
  Pop $R7
  Pop $R6
  Pop $R5
  Pop $R2
  Pop $R1
  Pop $R4
  Exch $R3
FunctionEnd